var Sequencing = function()
{
	var sequencing = {};
	var debug = 0;
	
	var turnAroundDateIsValid = false;
	var endDateIsValid = false;

	// Page initialization
	sequencing.initPage = function()
	{
		// Step 1
		Events.addEventHandler('seqRuns', 'change', sequencing.sequencingRunOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', sequencing.validateStep1);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', sequencing.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', sequencing.validateStep2);
		Events.addEventHandler('firstBaseReportOk', 'click', sequencing.turnAroundDateTimeOnChange);
		Events.addEventHandler('firstBaseReportFailed', 'click', sequencing.turnAroundDateTimeOnChange);
		Events.addEventHandler('turnAroundDate', 'blur', sequencing.turnAroundDateTimeOnChange);
		Events.addEventHandler('turnAroundTime', 'blur', sequencing.turnAroundDateTimeOnChange);
		Events.addEventHandler('endDate', 'blur', sequencing.endDateTimeOnChange);
		Events.addEventHandler('endDate', 'blur', sequencing.endDateTimeOnChange);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', sequencing.submit);
				
		Wizard.showLoadingAnimation('Loading sequencing runs...');	
		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=GetActiveSequencingRuns';
		Wizard.asyncJsonRequest(url, sequencing.initializeStep1);
	}

	
	sequencing.initializeStep1 = function(response)
	{
		var seqRuns = response.sequencingRuns;
		var frm = document.forms['reggie'];	

		if (seqRuns.length > 0)
		{
			for (var runNo=0; runNo < seqRuns.length; runNo++)
			{
				var seqRun = seqRuns[runNo];
				var option = sequencing.getOptionForSeqRun(seqRun);
				frm.seqRuns.options[frm.seqRuns.length] = option;
			}
		}
		else
		{
			Wizard.setFatalError('No sequencing runs available for processing.');
			return;
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		sequencing.sequencingRunOnChange();
		frm.seqRuns.focus();
	}
	
	sequencing.getOptionForSeqRun = function(seqRun)
	{
		var flowCell = seqRun.flowCell;
		var name = seqRun.name+': ';
		var numPools = flowCell.pools.length;
		var firstPoolNum = sequencing.getPoolNum(flowCell.pools[0].name);
		var lastPoolNum = sequencing.getPoolNum(flowCell.pools[flowCell.pools.length-1].name);

		if (lastPoolNum - firstPoolNum == numPools - 1)
		{
			if (numPools > 1)
			{
				// Display: PoolN -- PoolY
				name += flowCell.pools[0].name + ' — ' + flowCell.pools[flowCell.pools.length-1].name;
			}
			else
			{
				name += flowCell.pools[0].name;
			}
		}
		else
		{
			// Display: PoolN + x more...
			name += flowCell.pools[0].name + ' + ' + (numPools-1) + ' more...';
		}

		// Tooltip is always all pools
		var title = '';
		for (var poolNo=0; poolNo < numPools; poolNo++)
		{
			var pool = flowCell.pools[poolNo];
			if (poolNo > 0) title += ', ';
			title += pool.name;
		}
		
		if (flowCell.FlowCellType)
		{
			name += ' [' + Strings.encodeTags(flowCell.FlowCellType) + ']';
		}

		var option = new Option(name, seqRun.id);
		option.title = title;
		option.seqRun = seqRun;	
		return option;
	}

		
	sequencing.getPoolNum = function(poolName)
	{
		var num = poolName.match(/Pool(\d+)/);
		return num ? parseInt(num[1], 10) : null;
	}


	sequencing.sequencingRunOnChange = function()
	{
		var frm = document.forms['reggie'];	
		var seqRun = frm.seqRuns[frm.seqRuns.selectedIndex].seqRun;
		
		Doc.element('startDate').innerHTML = Reggie.reformatDate(seqRun.SequencingStart);
		
		var fc = seqRun.flowCell;
		Doc.element('flowCellsInRun').innerHTML = '<b>['+seqRun.HiSeqPosition+']</b> ' + Strings.encodeTags(fc.FlowCellID);
		Doc.element('flowCellType').innerHTML = Strings.encodeTags(fc.FlowCellType || '');
		var pools = fc.pools;
		var html = '';
		var libPlateHtml = '';
		for (var poolNo = 0; poolNo < pools.length; poolNo++)
		{
			var pool = pools[poolNo];
			if (poolNo > 0) html += ', ';
			html += Strings.encodeTags(pool.name);
			if (pool.LibPlate && libPlateHtml.indexOf(pool.LibPlate.name) == -1)
			{
				if (libPlateHtml.length > 0) libPlateHtml += ', ';
				libPlateHtml += Strings.encodeTags(pool.LibPlate.name);
			}
		}
		Doc.element('pools').innerHTML = html;
		Doc.element('libPlate').innerHTML = libPlateHtml || 'n/a';
		Doc.element('comments').innerHTML = Strings.encodeTags(seqRun.comments);
	}

	// Always ok
	sequencing.validateStep1 = function(event)
	{}
	
	sequencing.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		
		
		var seqRun = frm.seqRuns[frm.seqRuns.selectedIndex].seqRun;
		frm.sequencingComments.value = seqRun.comments;
		Doc.element('step-2-title').innerHTML = Strings.encodeTags(frm.seqRuns[frm.seqRuns.selectedIndex].text);

		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('goregister');

		frm.turnAroundDate.focus();
	}
	
	sequencing.turnAroundDateTimeOnChange = function()
	{
		var frm = document.forms['reggie'];
		turnAroundDateIsValid = false;
		Wizard.setInputStatus('turnAroundDate');
		
		var turnAroundDate = frm.turnAroundDate.value;
		var turnAroundTime = frm.turnAroundTime.value;
		
		if (turnAroundDate == '')
		{
			if (Doc.element('firstBaseReportOk').checked)
			{
				Wizard.setInputStatus('turnAroundDate', 'warning', 'Missing date');
			}
			turnAroundDateIsValid = true;
			return;
		}
		else
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			turnAroundDate = Reggie.autoFillDate(turnAroundDate);
			frm.turnAroundDate.value = turnAroundDate;

			if (!Dates.isDate(turnAroundDate, 'yyyyMMdd'))
			{
				Wizard.setInputStatus('turnAroundDate', 'invalid', 'Not a valid date');
				return;
			}
		}
		
		if (turnAroundTime == '')
		{
			Wizard.setInputStatus('turnAroundDate', 'invalid', 'Missing time');
			return;
		}
		else
		{
			turnAroundTime = Reggie.autoFillTime(turnAroundTime);
			frm.turnAroundTime.value = turnAroundTime;
			
			if (!Dates.isDate(turnAroundDate + ' ' + turnAroundTime, 'yyyyMMdd HHmm'))
			{
				Wizard.setInputStatus('turnAroundDate', 'invalid', 'Not a valid time');
				return;
			}
		}
		
		Wizard.setInputStatus('turnAroundDate', 'valid');
		turnAroundDateIsValid = true;
	}

	sequencing.endDateTimeOnChange = function()
	{
		var frm = document.forms['reggie'];
		endDateIsValid = false;
		Wizard.setInputStatus('endDate');
		
		var endDate = frm.endDate.value;
		var endTime = frm.endTime.value;
		
		if (endDate == '')
		{
			Wizard.setInputStatus('endDate', 'invalid', 'Missing date');
			return;
		}
		else
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			endDate = Reggie.autoFillDate(endDate);
			frm.endDate.value = endDate;

			if (!Dates.isDate(endDate, 'yyyyMMdd'))
			{
				Wizard.setInputStatus('endDate', 'invalid', 'Not a valid date');
				return;
			}
		}
		
		if (endTime == '')
		{
			Wizard.setInputStatus('endDate', 'invalid', 'Missing time');
			return;
		}
		else
		{
			endTime = Reggie.autoFillTime(endTime);
			frm.endTime.value = endTime;
			
			if (!Dates.isDate(endDate + ' ' + endTime, 'yyyyMMdd HHmm'))
			{
				Wizard.setInputStatus('endDate', 'invalid', 'Not a valid time');
				return;
			}
		}

		if (turnAroundDateIsValid)
		{
			var turnAroundTimestamp = Dates.parseString(frm.turnAroundDate.value + ' ' + frm.turnAroundTime.value, 'yyyyMMdd Hmm');
			var endTimestamp = Dates.parseString(frm.endDate.value + ' ' + frm.endTime.value, 'yyyyMMdd Hmm');
			
			if (turnAroundTimestamp != null && turnAroundTimestamp.getTime() >= endTimestamp.getTime())
			{
				Wizard.setInputStatus('endDate', 'invalid', 'End date must be after turnaround date');
				return;
			}
		}

		Wizard.setInputStatus('endDate', 'valid');
		endDateIsValid = true;
	}

	sequencing.validateStep2 = function(event)
	{
		if (!turnAroundDateIsValid) sequencing.turnAroundDateTimeOnChange();
		if (!endDateIsValid) sequencing.endDateTimeOnChange();
		if (!turnAroundDateIsValid || !endDateIsValid) event.preventDefault();
	}

	
	sequencing.submit = function()
	{
		
		var frm = document.forms['reggie'];
		var seqRun = frm.seqRuns[frm.seqRuns.selectedIndex].seqRun;
		seqRun.firstBaseReportFailed = Doc.element('firstBaseReportFailed').checked;
		seqRun.turnAroundDate = Strings.trim(frm.turnAroundDate.value + ' ' + frm.turnAroundTime.value);
		seqRun.endDate = Strings.trim(frm.endDate.value + ' ' + frm.endTime.value);
		seqRun.comments = frm.sequencingComments.value;
			
		var submitInfo = {};
		submitInfo.sequencingRun = seqRun;
			
		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterSequencingEnded';
			
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, sequencing.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	sequencing.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return sequencing;
}();

Doc.onLoad(Sequencing.initPage);

